@@ -32,7 +32,6 @@
 #include <pulsecore/core-util.h>
 #include <pulsecore/dbus-shared.h>
 
-#include "module-bluez4-discover-symdef.h"
 #include "bluez4-util.h"
 
 PA_MODULE_AUTHOR("João Paulo Rechi Vita");
@@ -93,7 +92,7 @@ static pa_hook_result_t load_module_for_device(pa_bluez4_discovery *y, const pa_
             }
 
             pa_log_debug("Loading module-bluez4-device %s", args);
-            m = pa_module_load(u->module->core, "module-bluez4-device", args);
+            pa_module_load(&m, u->module->core, "module-bluez4-device", args);
             pa_xfree(args);
 
             if (m) {
@@ -124,7 +123,7 @@ static pa_hook_result_t load_module_for_device(pa_bluez4_discovery *y, const pa_
 
 int pa__init(pa_module* m) {
     struct userdata *u;
-    pa_modargs *ma = NULL;
+    pa_modargs *ma;
 
     pa_assert(m);
 
@@ -140,7 +139,6 @@ int pa__init(pa_module* m) {
     u->module = m;
     u->core = m->core;
     u->modargs = ma;
-    ma = NULL;
     u->hashmap = pa_hashmap_new(pa_idxset_string_hash_func, pa_idxset_string_compare_func);
 
     if (!(u->discovery = pa_bluez4_discovery_get(u->core)))
@@ -154,9 +152,6 @@ int pa__init(pa_module* m) {
 fail:
     pa__done(m);
 
-    if (ma)
-        pa_modargs_free(ma);
-
     return -1;
 }
 
