@@ -62,8 +62,6 @@
 #include <pulsecore/thread.h>
 #include <pulsecore/time-smoother.h>
 
-#include "module-solaris-symdef.h"
-
 PA_MODULE_AUTHOR("Pierre Ossman");
 PA_MODULE_DESCRIPTION("Solaris Sink/Source");
 PA_MODULE_VERSION(PACKAGE_VERSION);
@@ -350,7 +348,7 @@ static int open_audio_device(struct userdata *u, pa_sample_spec *ss) {
     return u->fd;
 }
 
-static int suspend(struct userdata *u) {
+static void suspend(struct userdata *u) {
     pa_assert(u);
     pa_assert(u->fd >= 0);
 
@@ -366,8 +364,6 @@ static int suspend(struct userdata *u) {
     }
 
     pa_log_info("Device suspended.");
-
-    return 0;
 }
 
 static int unsuspend(struct userdata *u) {
@@ -394,54 +390,64 @@ static int sink_process_msg(pa_msgobject *o, int code, void *data, int64_t offse
         case PA_SINK_MESSAGE_GET_LATENCY:
             *((int64_t*) data) = sink_get_latency(u, &PA_SINK(o)->sample_spec);
             return 0;
-
-        case PA_SINK_MESSAGE_SET_STATE:
-
-            switch ((pa_sink_state_t) PA_PTR_TO_UINT(data)) {
-
-                case PA_SINK_SUSPENDED:
-
-                    pa_assert(PA_SINK_IS_OPENED(u->sink->thread_info.state));
-
-                    pa_smoother_pause(u->smoother, pa_rtclock_now());
-
-                    if (!u->source || u->source_suspended) {
-                        if (suspend(u) < 0)
-                            return -1;
-                    }
-                    u->sink_suspended = true;
-                    break;
-
-                case PA_SINK_IDLE:
-                case PA_SINK_RUNNING:
-
-                    if (u->sink->thread_info.state == PA_SINK_SUSPENDED) {
-                        pa_smoother_resume(u->smoother, pa_rtclock_now(), true);
-
-                        if (!u->source || u->source_suspended) {
-                            bool mute;
-                            if (unsuspend(u) < 0)
-                                return -1;
-                            u->sink->get_volume(u->sink);
-                            if (u->sink->get_mute(u->sink, &mute) >= 0)
-                                pa_sink_set_mute(u->sink, mute, false);
-                        }
-                        u->sink_suspended = false;
-                    }
-                    break;
-
-                case PA_SINK_INVALID_STATE:
-                case PA_SINK_UNLINKED:
-                case PA_SINK_INIT:
-                    ;
-            }
-
-            break;
     }
 
     return pa_sink_process_msg(o, code, data, offset, chunk);
 }
 
+/* Called from the IO thread. */
+static int sink_set_state_in_io_thread_cb(pa_sink *s, pa_sink_state_t new_state, pa_suspend_cause_t new_suspend_cause) {
+    struct userdata *u;
+
+    pa_assert(s);
+    pa_assert_se(u = s->userdata);
+
+    /* It may be that only the suspend cause is changing, in which case there's
+     * nothing to do. */
+    if (new_state == s->thread_info.state)
+        return 0;
+
+    switch (new_state) {
+
+        case PA_SINK_SUSPENDED:
+
+            pa_assert(PA_SINK_IS_OPENED(u->sink->thread_info.state));
+
+            pa_smoother_pause(u->smoother, pa_rtclock_now());
+
+            if (!u->source || u->source_suspended)
+                suspend(u);
+
+            u->sink_suspended = true;
+            break;
+
+        case PA_SINK_IDLE:
+        case PA_SINK_RUNNING:
+
+            if (u->sink->thread_info.state == PA_SINK_SUSPENDED) {
+                pa_smoother_resume(u->smoother, pa_rtclock_now(), true);
+
+                if (!u->source || u->source_suspended) {
+                    bool mute;
+                    if (unsuspend(u) < 0)
+                        return -1;
+                    u->sink->get_volume(u->sink);
+                    if (u->sink->get_mute(u->sink, &mute) >= 0)
+                        pa_sink_set_mute(u->sink, mute, false);
+                }
+                u->sink_suspended = false;
+            }
+            break;
+
+        case PA_SINK_INVALID_STATE:
+        case PA_SINK_UNLINKED:
+        case PA_SINK_INIT:
+            ;
+    }
+
+    return 0;
+}
+
 static int source_process_msg(pa_msgobject *o, int code, void *data, int64_t offset, pa_memchunk *chunk) {
     struct userdata *u = PA_SOURCE(o)->userdata;
 
@@ -450,48 +456,58 @@ static int source_process_msg(pa_msgobject *o, int code, void *data, int64_t off
         case PA_SOURCE_MESSAGE_GET_LATENCY:
             *((pa_usec_t*) data) = source_get_latency(u, &PA_SOURCE(o)->sample_spec);
             return 0;
-
-        case PA_SOURCE_MESSAGE_SET_STATE:
-
-            switch ((pa_source_state_t) PA_PTR_TO_UINT(data)) {
-
-                case PA_SOURCE_SUSPENDED:
-
-                    pa_assert(PA_SOURCE_IS_OPENED(u->source->thread_info.state));
-
-                    if (!u->sink || u->sink_suspended) {
-                        if (suspend(u) < 0)
-                            return -1;
-                    }
-                    u->source_suspended = true;
-                    break;
-
-                case PA_SOURCE_IDLE:
-                case PA_SOURCE_RUNNING:
-
-                    if (u->source->thread_info.state == PA_SOURCE_SUSPENDED) {
-                        if (!u->sink || u->sink_suspended) {
-                            if (unsuspend(u) < 0)
-                                return -1;
-                            u->source->get_volume(u->source);
-                        }
-                        u->source_suspended = false;
-                    }
-                    break;
-
-                case PA_SOURCE_UNLINKED:
-                case PA_SOURCE_INIT:
-                case PA_SOURCE_INVALID_STATE:
-                    ;
-
-            }
-            break;
-
     }
 
     return pa_source_process_msg(o, code, data, offset, chunk);
 }
 
+/* Called from the IO thread. */
+static int source_set_state_in_io_thread_cb(pa_source *s, pa_source_state_t new_state, pa_suspend_cause_t new_suspend_cause) {
+    struct userdata *u;
+
+    pa_assert(s);
+    pa_assert_se(u = s->userdata);
+
+    /* It may be that only the suspend cause is changing, in which case there's
+     * nothing to do. */
+    if (new_state == s->thread_info.state)
+        return 0;
+
+    switch (new_state) {
+
+        case PA_SOURCE_SUSPENDED:
+
+            pa_assert(PA_SOURCE_IS_OPENED(u->source->thread_info.state));
+
+            if (!u->sink || u->sink_suspended)
+                suspend(u);
+
+            u->source_suspended = true;
+            break;
+
+        case PA_SOURCE_IDLE:
+        case PA_SOURCE_RUNNING:
+
+            if (u->source->thread_info.state == PA_SOURCE_SUSPENDED) {
+                if (!u->sink || u->sink_suspended) {
+                    if (unsuspend(u) < 0)
+                        return -1;
+                    u->source->get_volume(u->source);
+                }
+                u->source_suspended = false;
+            }
+            break;
+
+        case PA_SOURCE_UNLINKED:
+        case PA_SOURCE_INIT:
+        case PA_SOURCE_INVALID_STATE:
+            ;
+
+    }
+
+    return 0;
+}
+
 static void sink_set_volume(pa_sink *s) {
     struct userdata *u;
     audio_info_t info;
@@ -966,6 +982,7 @@ int pa__init(pa_module *m) {
 
         u->source->userdata = u;
         u->source->parent.process_msg = source_process_msg;
+        u->source->set_state_in_io_thread = source_set_state_in_io_thread_cb;
 
         pa_source_set_asyncmsgq(u->source, u->thread_mq.inq);
         pa_source_set_rtpoll(u->source, u->rtpoll);
@@ -1009,6 +1026,7 @@ int pa__init(pa_module *m) {
         pa_assert(u->sink);
         u->sink->userdata = u;
         u->sink->parent.process_msg = sink_process_msg;
+        u->sink->set_state_in_io_thread = sink_set_state_in_io_thread_cb;
 
         pa_sink_set_asyncmsgq(u->sink, u->thread_mq.inq);
         pa_sink_set_rtpoll(u->sink, u->rtpoll);
