@@ -62,12 +62,6 @@
 #include <pulsecore/x11prop.h>
 #endif
 
-#ifdef TUNNEL_SINK
-#include "module-tunnel-sink-symdef.h"
-#else
-#include "module-tunnel-source-symdef.h"
-#endif
-
 #define ENV_DEFAULT_SINK "PULSE_SINK"
 #define ENV_DEFAULT_SOURCE "PULSE_SOURCE"
 #define ENV_DEFAULT_SERVER "PULSE_SERVER"
@@ -574,11 +568,16 @@ static int sink_process_msg(pa_msgobject *o, int code, void *data, int64_t offse
 }
 
 /* Called from main context */
-static int sink_set_state(pa_sink *s, pa_sink_state_t state) {
+static int sink_set_state_in_main_thread_cb(pa_sink *s, pa_sink_state_t state, pa_suspend_cause_t suspend_cause) {
     struct userdata *u;
     pa_sink_assert_ref(s);
     u = s->userdata;
 
+    /* It may be that only the suspend cause is changing, in which
+     * case there's nothing to do. */
+    if (state == s->state)
+        return 0;
+
     switch ((pa_sink_state_t) state) {
 
         case PA_SINK_SUSPENDED:
@@ -671,11 +670,16 @@ static int source_process_msg(pa_msgobject *o, int code, void *data, int64_t off
 }
 
 /* Called from main context */
-static int source_set_state(pa_source *s, pa_source_state_t state) {
+static int source_set_state_in_main_thread_cb(pa_source *s, pa_source_state_t state, pa_suspend_cause_t suspend_cause) {
     struct userdata *u;
     pa_source_assert_ref(s);
     u = s->userdata;
 
+    /* It may be that only the suspend cause is changing, in which
+     * case there's nothing to do. */
+    if (state == s->state)
+        return 0;
+
     switch ((pa_source_state_t) state) {
 
         case PA_SOURCE_SUSPENDED:
@@ -2152,7 +2156,7 @@ int pa__init(pa_module*m) {
 
     u->sink->parent.process_msg = sink_process_msg;
     u->sink->userdata = u;
-    u->sink->set_state = sink_set_state;
+    u->sink->set_state_in_main_thread = sink_set_state_in_main_thread_cb;
     pa_sink_set_set_volume_callback(u->sink, sink_set_volume);
     pa_sink_set_set_mute_callback(u->sink, sink_set_mute);
 
@@ -2195,7 +2199,7 @@ int pa__init(pa_module*m) {
     }
 
     u->source->parent.process_msg = source_process_msg;
-    u->source->set_state = source_set_state;
+    u->source->set_state_in_main_thread = source_set_state_in_main_thread_cb;
     u->source->userdata = u;
 
 /*     pa_source_set_latency_range(u->source, MIN_NETWORK_LATENCY_USEC, 0); */
