@@ -470,7 +470,7 @@ static ssize_t send_udp_audio_packet(pa_raop_client *c, pa_memchunk *block, size
     buffer += packet->index;
     if (buffer && packet->length > 0)
         written = pa_write(c->udp_sfd, buffer, packet->length, NULL);
-    if (written < 0 && (errno == EAGAIN || errno == EWOULDBLOCK)) {
+    if (written < 0 && errno == EAGAIN) {
         pa_log_debug("Discarding UDP (audio, seq=%d) packet due to EAGAIN (%s)", c->seq, pa_cstrerror(errno));
         written = packet->length;
     }
@@ -525,8 +525,8 @@ static ssize_t resend_udp_audio_packets(pa_raop_client *c, uint16_t seq, uint16_
 
         if (buffer && packet->length > 0)
             written = pa_write(c->udp_cfd, buffer, packet->length, NULL);
-        if (written < 0 && (errno == EAGAIN || errno == EWOULDBLOCK)) {
-            pa_log_debug("Discarding UDP (audio-restransmitted, seq=%d) packet due to EAGAIN", seq + i);
+        if (written < 0 && errno == EAGAIN) {
+            pa_log_debug("Discarding UDP (audio-retransmitted, seq=%d) packet due to EAGAIN", seq + i);
             pa_memblock_release(packet->memblock);
             continue;
         }
